<?php
/*
 *   This file is part of CLIGS v 2.0 - Cute Little Image Gallery System
 *   (c) Copyright 2005 John Leach <john@johnleach.co.uk> 
 *   http://johnleach.co.uk/documents/cligs
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

require_once 'sets.php';
require_once 'image.php';
require_once 'html.php';

define('MODE_INFO', 'info');
define('MODE_FULL', 'full');
define('MODE_THUMB', 'thumb');
define('MODE_SMALLTHUMB', 'smallthumb');
define('MODE_PREVIEW', 'preview');
define('MODE_ICON', 'icon');
define('MODE_BIG', 'big');

$cligs_errorstack = array();

function cligs_errorhandler($errLvl, $errMsg, $errFile, $errLine, $errContext)
{
	global $cligs_errorstack;
	// Switch statement watching for specific error levels
	switch ( $errLvl ) {
	case E_USER_WARNING:
	case E_USER_NOTICE:
		$cligs_errorstack[] = $errMsg;
		break;
	case E_USER_ERROR:
		$error_title = $errMsg;
		include 'cligs-error.php';
		die();
		break;
	default:
		#echo "<li>$errMsg;";
		break;
	}
}

class cligs
{

# The Metadata object
var $meta;
# Array of gallery root paths
var $roots;
var $errorstack;


/* Output an error page */
function error( $error_mesg )
{
	$error_title = 'Error';
	include $this->theme_path . '/error_view.php';
}


/* Given a filesystem path, construct a link for use in <a href */
/* detects if using url rewriting or cgi variables */
function make_url( $path, $mode=FALSE )
{
	if ( isset( $_SERVER['REDIRECT_QUERY_STRING'] ) && 
			eregi('path=', $_SERVER['REDIRECT_QUERY_STRING']) ) {
		// Apache redirect mode
		$url = str_replace($_SERVER['DOCUMENT_ROOT'], '', $path);
		if ($mode)
			$url = $url . "?$mode";
	} else {
		// GET string mode
		$url = '?path=' . str_replace($_SERVER['DOCUMENT_ROOT'], '', $path);
		if ($mode)
			$url = $url . "&$mode";
	}

	$url = str_replace("//", '/', $url);
	return $url;
}


function precache_thumbs( )
{

}


/* Draw the html for a directory set */
function display_directory()
{
	$set = new set_filesystem( $this->path );
	$set->load();
	$this->meta->title = $set->meta->title;
		
	if ( $this->mode == MODE_SMALLTHUMB ) {
		/* choose an image from the dir */
		if ( $set->meta->icon != FALSE) {
			$imgpath = $this->path . '/' . $set->meta->icon;
		} else {
			$imgcount = count( $set->images );
			if ( $imgcount == 0 ) {
				header('Location: ' . $this->make_url($this->theme_path . '/noicon.png'));
				return TRUE;
			}
			srand( $imgcount ); // Choose the same img each time
			$imgpath = $set->images[rand( 0, $imgcount - 1 )]->path;
		}
		$imgproc = new image_processor( $this->cache_path );
		$cache_filename = $imgproc->resize_image( $imgpath, MODE_SMALLTHUMB );
		return $imgproc->send_image( $cache_filename );
	}

	include $this->theme_path . '/set_view.php';
}


/* Handle a request for a file.  Either outputs an image or a image info page */
function display_file()
{
	if ( $this->mode == MODE_INFO ) {
		$set = new set_filesystem( $this->path );
		$set->load();
		$this->meta->title = $set->meta->title . " &raquo; " . $set->thisimg->meta->title;
		
		include $this->theme_path . '/image_info_view.php';
		return TRUE;
	}

	$imgproc = new image_processor( $this->cache_path );
	$cache_filename = $imgproc->resize_image( $this->path, $this->mode );
	$imgproc->send_image( $cache_filename );
}


/* Create and display a set based on the cligs roots config */
function display_root_list()
{
	$set = new set_from_root_list( $this->roots );

	$set->meta = new Metadata;

	$set->meta->title = 'Cligs &raquo; Image Set List';
	$set->meta->root = TRUE;
	$this->meta->title = $set->meta->title;

	include $this->theme_path . '/set_view.php';
}


/* Decide what type of request this is and process it*/
function display()
{
	$rootok = FALSE;
	foreach ( $this->roots as $root )
		if ( eregi( "^$root", $this->path ) )
			$rootok = TRUE;

	if ( !$rootok || !$this->path )
		return $this->display_root_list();
			
	if ( is_dir( $this->path ) )
		return $this->display_directory();

	if ( is_file( $this->path ) )
		return $this->display_file();

	# FIXME: 404 here
	trigger_error( 'Path not found or inaccessible', E_USER_ERROR );
}


/* Constructor */
function cligs()
{
	$this->errorstack = array();
	
	$this->meta = new Metadata;

	$this->mode = MODE_BIG;
	if ( isset( $_GET['icon'] ) )
		$this->mode = MODE_ICON;
	elseif ( isset( $_GET['thumb'] ) )
		$this->mode = MODE_THUMB;
	elseif ( isset( $_GET['preview'] ) )
		$this->mode = MODE_PREVIEW;
	elseif ( isset( $_GET['full'] ) )
		$this->mode = MODE_FULL;
	elseif ( isset( $_GET['info'] ) )
		$this->mode = MODE_INFO;
	elseif ( isset( $_GET['smallthumb'] ) )
		$this->mode = MODE_SMALLTHUMB;
	
	$root = $_SERVER['DOCUMENT_ROOT'];
	$this->path = isset($_GET['path']) ? $root . '/' . $_GET['path'] : FALSE; # FIXME user input

	$this->theme_path = dirname( $_SERVER['SCRIPT_FILENAME'] ) . '/themes/default';
	$this->theme_uri = dirname( $_SERVER['SCRIPT_NAME'] ) . '/themes/default';
	$this->cache_path = dirname( $_SERVER['SCRIPT_FILENAME'] ) . '/cligsdata/imagecache';

	$this->roots = array();
	include 'cligs-config.php';
}

}

?>
