<?php

class image_processor
{

var $cachedir = '/tmp'; # default setting

var $sizes = array (
	MODE_THUMB	=> array( 160, 180, 90 ),
	MODE_SMALLTHUMB => array( 100, 110, 90 ),
	MODE_PREVIEW	=> array( 450, 0, 95 ),
	MODE_ICON	=> array( 45, 35, 90 ),
	MODE_FULL	=> array( 0, 0, 0 ),
	MODE_BIG	=> array( 1024, 0, 90 )
	);

function image_processor( $cachedir )
{
	if (! ereg('.*/$', $cachedir ) ) # FIXME: ugh, ereg too much
		$cachedir = $cachedir . '/';

	$this->cachedir = $cachedir;
}

/* Output last modified header using file modification time */
function lastmod( $filename )
{
        header( "Last-Modified: " . @date( "D, d M Y H:i:s", @filemtime( $filename ) )  . " GMT" );
}

/* return a absolute filesystem path to the filename in the image cache
 * based on the name, width, height and quality */
function cache_file_name( $filename, $width, $height, $quality )
{
	$filename = realpath($filename);
        $path = $this->cachedir . md5( dirname( $filename ) );
	$cache_file_name = $path . '/' . "$width-$height-$quality-" . basename( $filename );
	if ( !@is_dir( $path ) ) {
		if ( !mkdir( $path ) ) {
			trigger_error( 'Could not create cache directory', E_USER_ERROR );
		}
	}
	return $cache_file_name;
}

/* send jpeg, png or gif image to browser with appropriate http headers */
function send_image( $filename )
{
        $this->lastmod( $filename );
        # Not cached so generate a new thumb
	list($ow, $oh, $type, $attr) = getimagesize( $filename );
	switch( $type ) {
		case 3:
        		header("Content-type: image/png");
			break;
		case 2:
        		header("Content-type: image/jpg");
			break;
		case 1:
        		header("Content-type: image/gif");
			break;
	}
        return readfile( $filename );
}

/* Open, resize and cache an image (currently only png, gif or jpeg)
 * return TRUE or FALSE */
function resize_image( $filename, $mode, $logo=false )
{
	# MODE_FULL, no resizing
	if ( $mode == MODE_FULL )
		return $filename;

	/* get size and quality from sizes array keyed on mode */ 
	$imgsize = $this->sizes[ $mode ];
	$w = $imgsize[ 0 ];
	$h = $imgsize[ 1 ];
	$q = $imgsize[ 2 ];

	$thumbname = $this->cache_file_name( $filename, $w, $h, $q );

        # Check the cache
        if ( file_exists( $thumbname ) && 
		( filemtime( $filename ) < filemtime( $thumbname ) ) ) {
		/* FIXME: and $logo mtime check */
		return $thumbname;
        }

        # Not cached so generate a new thumb

	# detect image type and create gd object
	list($ow, $oh, $type, $attr) = getimagesize( $filename );
	switch( $type ) {
		case 3:
			$o = imagecreatefrompng( $filename );
			break;
		case 2:
			$o = imagecreatefromjpeg( $filename );
			break;
		case 1:
			$o = imagecreatefromgif( $filename );
			break;
		default:
			trigger_error( 'That image format is not supported', E_USER_ERROR );
			break;
	}

        if ( $logo )
                $logopng = imagecreatefrompng( $logo );

	# if no height requested, make it a correct ratio of specified width
        if ( $h == 0 )
                $h = ( ( $w / $ow ) * $oh );
	# if no width requested, make it a correct ratio of specified height
        if ( $w == 0 )
                $w = ( ( $h / $oh ) * $ow );

        $nw = ( ($h / $oh ) * $ow );
        $nh = ( ($w / $ow ) * $oh );
       
	/* if original size is smaller than requeted size, then don't stretch */
	if ( ( $ow < $w ) and ( $oh < $h ) ) {
                $nw = $ow; $nh = $oh;
                $h = $oh; $w = $ow;
        }

	/* crop */
        if ( $nw > $w )
                $nh = $h;
	
        if ($nh > $h)
                $nw = $w;

        # If aspect ratio isn't preserved, try to center the image
        $y = 0;
        if ( $nh > $h )
                $y = ( $nh - $h ) / 2;
        
	$x = 0;
	if ( $nw > $w )
                $x = ( $nw - $w ) / 2;
	
        $resize = ImageCreateTrueColor( $nw, $nh );
        $thumb = ImageCreateTrueColor( $w, $h );

        imagecopyresampled( $resize, $o, 0, 0, 0, 0, $nw, $nh, $ow, $oh );
        imagedestroy( $o );
        imagecopy( $thumb, $resize, 0, 0, $x, $y, $w, $h );

        if ( isset( $logopng ) )
                imagecopy( $thumb, $logopng, 0, 0, 0, $y, imageSX( $logopng ), imageSY( $logopng ) );

        imagedestroy( $resize );

        # Save to cache
	switch( $type ) {
		case 3:
        		imagepng( $thumb, $thumbname, $q );
			break;
		case 2:
        		imagejpeg( $thumb, $thumbname, $q );
			break;
		case 1:
        		imagegif( $thumb, $thumbname, $q );
			break;
		default:
			# error checking above prevents ever getting here
			break;
	}

        imagedestroy( $thumb );

        return is_readable( $thumbname ) ? $thumbname : FALSE;
}

}

