<?php
/* 
 *   This file is part of CLIGS v 2.0 - Cute Little Image Gallery System
 *   (c) Copyright 2005 John Leach <john@johnleach.co.uk> 
 *   http://johnleach.co.uk/documents/cligs
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

require_once 'metadata.php';
require_once 'set_filesystem.php';

class image
{
	var $path;
	var $filename;
	var $stat;
	var $title;
	
	function image( $path )
	{
		$this->path = $path;
		$this->filename = basename( $path );
		$this->stat = stat( $path );
		$this->meta = new metadata_from_file( $path );
	}
}

class Set
{
	var $subsets = array();
	var $images = array();
	var $imglistcount;
	# Subset representing this set's parent
	var $parent;
	# Metadata object;
	var $meta;
	var $thisimg; # currently selected image
	var $previmg; # previous image in the set
	var $nextimg; # next image in the set
	
	function add_subset( &$set )
	{
		$this->subsets[] = $set;
		return;
	}

	function add_image( &$img )
	{
		$this->images[] = $img;
		$this->meta->tags = array_merge($this->meta->tags, $img->meta->tags);
		$this->meta->tags = array_unique($this->meta->tags);
		return;
	}

	# return a string representing the range of dates of images in the set
	function get_date_range()
	{
		if ( count($this->images) == 0 )
			return '';
		$max = 0;
		$min = 9999999999;
		foreach ( $this->images as $image ) {
			if ( $image->meta->time == 0 )
				continue;
			$max = max( $image->meta->time, $max );
			$min = min( $image->meta->time, $min );
		}
		if ( $max == 0 || $min == 9999999999 )
			return FALSE;

		if ( date('dmy', $min) == date('dmy', $max) )
			return date('l jS F Y', $max);

		if ( date('MY', $min) == date('MY', $max) )
			return date('jS -', $min) . date('jS F Y', $max);
		
		if ( date('Y', $min) == date('Y', $max) )
			return date('jS F -', $min) . date('jS F Y', $max);

		return date('jS F Y - ', $min) . date('jS F Y', $max);

	}
}

class set_from_root_list extends Set
{
	# Filesystem path of this set
	var $path;
	# Filename FIXME
	var $filename;
	
	function set_from_root_list( $roots )
	{
		foreach ( $roots as $root ) {
			if ( is_dir( $root ) ) {
				$subset = new set_filesystem( $root );
				$this->add_subset( $subset );
			}
		}

		$this->parent = new Set( );
		$this->parent->meta = new Metadata;
		$this->parent->meta->title = 'Cligs';
		$this->parent->path = FALSE;

		return TRUE;
	}
}
	
?>
